#!/bin/sh
# the next line restarts using the interpreter \
exec wish "$0" "$@"
# use wish instead of tclsh in order to handle graphical packages

set rcsId {$Id: mkpkgidx.tcl,v 1.4 1996/12/27 21:52:23 jfontain Exp $}

# this utility must be used to create the package index file for a package that uses stooop.
# since a base class constructor must be defined before a derived class constructor, the base class source file must be sourced
# before the derived class source file. the pkg_mkIndex utility procedure creates index files that do not guarantee any order in the
# sourcing of files. thus, as a new derived object is created, the base class source file(s) may have not been sourced yet, which
# results in an error for stooop.
# this utility fixes this problem by making sure class source files are sourced in the proper order, that is the base class source
# file(s) before any derived class source files.
# the trick is to invoke the base class constructor(s) with no arguments, catch the error, and the unknown procedure will take care
# of sourcing the required file, once only.

package require stooop

set packageName [lindex $argv 0]
set index "# Tcl package index file, version 1.0\n"
append index "# created with stooop version [package provide stooop] for stooop packages\n"
set indexData {}

foreach file [lrange $argv 1 end] {
    set originalCommands [info commands]
    source $file
    catch {unset newCommands}
    foreach command [info commands] {
        set newCommands($command) {}
    }
    foreach command $originalCommands {
        catch {unset newCommands($command)}
    }
    set newClasses {}
    foreach command [array names newCommands] {
        if {[regexp {^(.+)::(.+)$} $command dummy class procedure]&&([string compare $class $procedure]==0)} {
            # detect constructors
            lappend newClasses $class
            # keep track of source file
            set sourceFile($class) $file
        }
    }
    set requiredClasses {}
    foreach class $newClasses {
        foreach baseClass $_bases($class) {
            # avoid duplicate entries
            if {[lsearch -exact $requiredClasses $baseClass]>=0} continue
            # nothing to do if base class is in the same file
            if {[string compare $sourceFile($baseClass) $file]!=0} {
                lappend requiredClasses $baseClass
            }
        }
    }
    # so far only sourceable file, not shared libraries, are handled
    lappend indexData [list $file source $requiredClasses [lsort [array names newCommands]]]
}

set version [package provide $packageName]
append index "\npackage ifneeded $packageName $version \[list stooopPackageSetup \$dir $packageName $version [list $indexData]\]"

set file [open pkgIndex.tcl w]
puts $file $index
close $file

exit
